@@ -1,3 +1,26 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/*
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) version 3.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with the program; if not, see <http://www.gnu.org/licenses/>
+ *
+ *
+ * Authors:
+ *    Bharath Acharya <abharath@novell.com>
+ *
+ * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
+ *
+ */
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -21,6 +44,9 @@
 
 #include "e-book-backend-mapi-gal.h"
 
+/* default value for "partial-count", upper bound of objects to download during partial search */
+#define DEFAULT_PARTIAL_COUNT 50
+
 G_DEFINE_TYPE (EBookBackendMAPIGAL, e_book_backend_mapi_gal, E_TYPE_BOOK_BACKEND_MAPI)
 
 struct _EBookBackendMAPIGALPrivate
@@ -31,68 +57,37 @@ struct _EBookBackendMAPIGALPrivate
 	gint32 unused;
 };
 
-static gchar *
-get_uid_from_row (struct SRow *aRow, uint32_t row_index)
-{
-	gchar *suid = NULL;
-	const gchar *str;
-
-	g_return_val_if_fail (aRow != NULL, NULL);
-
-	str = e_mapi_util_find_row_propval (aRow, PR_EMAIL_ADDRESS_UNICODE);
-	if (str && *str)
-		suid = g_strdup (str);
-
-	if (!suid) {
-		const mapi_id_t *midptr;
-
-		midptr = e_mapi_util_find_row_propval (aRow, PR_MID);
-
-		suid = e_mapi_util_mapi_id_to_string (midptr ? *midptr : row_index);
-	}
-
-	return suid;
-}
-
-struct FetchGalData
+struct TransferGalData
 {
 	EBookBackendMAPI *ebma;
 	EDataBookView *book_view;
 	gpointer notify_contact_data;
-	mapi_id_t fid; /* folder ID of contacts, for named IDs */
 };
 
 static gboolean
-fetch_gal_cb (EMapiConnection *conn,
-	      uint32_t row_index,
-	      uint32_t n_rows,
-	      struct SRow *aRow,
-	      gpointer data,
-	      GCancellable *cancellable,
-	      GError **perror)
+transfer_gal_cb (EMapiConnection *conn,
+		 TALLOC_CTX *mem_ctx,
+		 /* const */ EMapiObject *object,
+		 guint32 obj_index,
+		 guint32 obj_total,
+		 gpointer user_data,
+		 GCancellable *cancellable,
+		 GError **perror)
 {
-	struct FetchGalData *fgd = data;
+	struct TransferGalData *tg = user_data;
 	EContact *contact;
 
 	g_return_val_if_fail (conn != NULL, FALSE);
-	g_return_val_if_fail (aRow != NULL, FALSE);
-	g_return_val_if_fail (data != NULL, FALSE);
+	g_return_val_if_fail (object != NULL, FALSE);
+	g_return_val_if_fail (tg != NULL, FALSE);
 
-	contact = mapi_book_utils_contact_from_props (conn, fgd->fid, e_book_backend_mapi_get_book_uri (fgd->ebma), NULL, aRow);
+	contact = e_mapi_book_utils_contact_from_object (conn, object, e_book_backend_mapi_get_book_uri (tg->ebma));
 	if (!contact) {
-		/* just ignore them */
+		/* this is GAL, just ignore them */
 		return TRUE;
 	}
 
-	if (!e_contact_get_const (contact, E_CONTACT_UID)) {
-		gchar *suid;
-
-		suid = get_uid_from_row (aRow, row_index);
-		e_contact_set (contact, E_CONTACT_UID, suid);
-		g_free (suid);
-	}
-
-	if (!e_book_backend_mapi_notify_contact_update (fgd->ebma, fgd->book_view, contact, row_index, n_rows, fgd->notify_contact_data)) {
+	if (!e_book_backend_mapi_notify_contact_update (tg->ebma, tg->book_view, contact, obj_index, obj_total, tg->notify_contact_data)) {
 		g_object_unref (contact);
 		return FALSE;
 	}
@@ -104,9 +99,10 @@ fetch_gal_cb (EMapiConnection *conn,
 
 static gboolean
 list_gal_uids_cb (EMapiConnection *conn,
-		  uint32_t row_index,
-		  uint32_t n_rows,
-		  struct SRow *aRow,
+		  TALLOC_CTX *mem_ctx,
+		  const ListObjectsData *object_data,
+		  guint32 obj_index,
+		  guint32 obj_total,
 		  gpointer user_data,
 		  GCancellable *cancellable,
 		  GError **perror)
@@ -115,24 +111,18 @@ list_gal_uids_cb (EMapiConnection *conn,
 	struct ListKnownUidsData *lku = user_data;
 
 	g_return_val_if_fail (conn != NULL, FALSE);
-	g_return_val_if_fail (aRow != NULL, FALSE);
+	g_return_val_if_fail (object_data != NULL, FALSE);
 	g_return_val_if_fail (lku != NULL, FALSE);
 
-	uid = get_uid_from_row (aRow, row_index);
+	uid = e_mapi_util_mapi_id_to_string (object_data->mid);
 	if (uid) {
-		const struct FILETIME *ft;
-		time_t tt;
+		if (lku->latest_last_modify < object_data->last_modified)
+			lku->latest_last_modify = object_data->last_modified;
 
-		ft = e_mapi_util_find_row_propval (aRow, PidTagLastModificationTime);
-		tt = ft ? e_mapi_util_filetime_to_time_t (ft) : 1;
-
-		if (lku->latest_last_modify < tt)
-			lku->latest_last_modify = tt;
-
-		g_hash_table_insert (lku->uid_to_rev, uid, mapi_book_utils_timet_to_string (tt));
+		g_hash_table_insert (lku->uid_to_rev, uid, e_mapi_book_utils_timet_to_string (object_data->last_modified));
 	}
 
-	return !g_cancellable_is_cancelled (cancellable);
+	return TRUE;
 }
 
 static void
@@ -180,10 +170,13 @@ ebbm_gal_transfer_contacts (EBookBackendMAPI *ebma,
 			    GError **error)
 {
 	GError *mapi_error = NULL;
-	struct FetchGalData fgd = { 0 };
+	struct TransferGalData tg = { 0 };
 	EMapiConnection *conn;
-	gchar *last_fetch;
-	gboolean fetch_successful;
+	ESource *source;
+	GSList *get_mids = NULL;
+	const GSList *iter;
+	gint partial_count = -1;
+	gboolean status;
 
 	e_book_backend_mapi_lock_connection (ebma);
 
@@ -194,50 +187,49 @@ ebbm_gal_transfer_contacts (EBookBackendMAPI *ebma,
 		return;
 	}
 
-	/* GAL doesn't use restrictions yet, thus just fetches all items always */
-	last_fetch = e_book_backend_mapi_cache_get (ebma, "gal-last-update");
-	if (last_fetch) {
-		GTimeVal last_tv = { 0 }, now = { 0 };
+	source = e_backend_get_source (E_BACKEND (ebma));
+	if (source &&
+	    !e_book_backend_mapi_is_marked_for_offline (ebma) &&
+	    g_strcmp0 (e_source_get_property (source, "allow-partial"), "true") == 0) {
+		const gchar *partial_count_str = e_source_get_property (source, "partial-count");
 
-		g_get_current_time (&now);
+		partial_count = 0;
+		if (partial_count_str)
+			partial_count = atoi (partial_count_str);
 
-		/* refetch gal only once per week */
-		if (g_time_val_from_iso8601 (last_fetch, &last_tv) && now.tv_sec - last_tv.tv_sec <= 60 * 60 * 24 * 7) {
-			g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_CANCELLED, "Cancelled");
-			g_free (last_fetch);
-			e_book_backend_mapi_unlock_connection (ebma);
-			return;
-		}
-
-		g_free (last_fetch);
+		if (partial_count <= 0)
+			partial_count = DEFAULT_PARTIAL_COUNT;
 	}
 
-	fgd.ebma = ebma;
-	fgd.book_view = book_view;
-	fgd.notify_contact_data = notify_contact_data;
-	fgd.fid = e_mapi_connection_get_default_folder_id (conn, olFolderContacts, NULL, NULL);
+	for (iter = uids; iter && (partial_count == -1 || partial_count > 0); iter = iter->next) {
+		mapi_id_t *pmid, mid;
 
-	fetch_successful = e_mapi_connection_fetch_gal (conn, NULL, NULL,
-		mapi_book_utils_get_prop_list, GET_ALL_KNOWN_IDS,
-		fetch_gal_cb, &fgd, NULL, &mapi_error);
+		if (e_mapi_util_mapi_id_from_string  (iter->data, &mid)) {
+			pmid = g_new0 (mapi_id_t, 1);
+			*pmid = mid;
+
+			get_mids = g_slist_prepend (get_mids, pmid);
+
+			if (partial_count > 0)
+				partial_count--;
+		}
+	}
+
+	tg.ebma = ebma;
+	tg.book_view = book_view;
+	tg.notify_contact_data = notify_contact_data;
+
+	status = e_mapi_connection_transfer_gal_objects	(conn, get_mids, transfer_gal_cb, &tg, cancellable, &mapi_error);
 
 	if (mapi_error) {
 		mapi_error_to_edb_error (error, mapi_error, E_DATA_BOOK_STATUS_OTHER_ERROR, _("Failed to fetch GAL entries"));
 		g_error_free (mapi_error);
-	} else if (!fetch_successful) {
+	} else if (!status) {
 		g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_CANCELLED, "Cancelled");
-	} else {
-		GTimeVal now = { 0 };
-
-		g_get_current_time (&now);
-
-		last_fetch = g_time_val_to_iso8601 (&now);
-		if (last_fetch && *last_fetch)
-			e_book_backend_mapi_cache_set (ebma, "gal-last-update", last_fetch);
-
-		g_free (last_fetch);
 	}
 
+	g_slist_free_full (get_mids, g_free);
+
 	e_book_backend_mapi_unlock_connection (ebma);
 }
 
@@ -247,11 +239,24 @@ ebbm_gal_get_contacts_count (EBookBackendMAPI *ebma,
 			     GCancellable *cancellable,
 			     GError **error)
 {
+	EMapiConnection *conn;
+
 	e_return_data_book_error_if_fail (ebma != NULL, E_DATA_BOOK_STATUS_INVALID_ARG);
 	e_return_data_book_error_if_fail (obj_total != NULL, E_DATA_BOOK_STATUS_INVALID_ARG);
 
-	/* just a fake value, to check by ids */
-	*obj_total = -1;
+	e_book_backend_mapi_lock_connection (ebma);
+
+	conn = e_book_backend_mapi_get_connection (ebma);
+	if (!conn) {
+		e_book_backend_mapi_unlock_connection (ebma);
+		g_propagate_error (error, EDB_ERROR (REPOSITORY_OFFLINE));
+		return;
+	}
+
+	if (!e_mapi_connection_count_gal_objects (conn, obj_total, cancellable, error))
+		*obj_total = -1;
+
+	e_book_backend_mapi_unlock_connection (ebma);
 }
 
 static void
@@ -278,9 +283,7 @@ ebbm_gal_list_known_uids (EBookBackendMAPI *ebma,
 		return;
 	}
 
-	e_mapi_connection_fetch_gal (conn, NULL, NULL,
-		mapi_book_utils_get_prop_list, GET_UIDS_ONLY,
-		list_gal_uids_cb, lku, cancellable, &mapi_error);
+	e_mapi_connection_list_gal_objects (conn, build_rs_cb, build_rs_cb_data, list_gal_uids_cb, lku, cancellable, &mapi_error);
 
 	if (mapi_error) {
 		mapi_error_to_edb_error (error, mapi_error, E_DATA_BOOK_STATUS_OTHER_ERROR, _("Failed to fetch GAL entries"));
